/***********************************************************************/
/*                                                                     */
/*  FILE        :proto.c                                                 */
/*  DATE        :26.12.08                                              */
/*  COPYRIGHT   :Copyright (C) 2008 by Thomas Hochgtz                 */
/*                                                                     */
/***********************************************************************/
              
#include "proto.h"
#include "proto_device.h"
#include "proto_computer.h"
#include "cbm.h"
#include "r8c.h"

struct tSettings		sett;

#define DO_ERROR( err ) { sett.error = err; LED_3 = 1; SEND_UART(PC_ERROR); SEND_UART(err); }

			      
void	InitProto()
{
	InitCBM();
	
	INIT_UART;
	
	aFuncs[TODOFUNC_DEV_RECEIVER] = &ReceiverDevice;
	aFuncs[TODOFUNC_DEV_SENDER] = &SenderDevice;
	aFuncs[TODOFUNC_DEV_NONE] = &LoopNone;
	aFuncs[TODOFUNC_COM_RECEIVER] = &ReceiverComputer;
	aFuncs[TODOFUNC_COM_SENDER] = &SenderComputer;
	aFuncs[TODOFUNC_COM_NONE] = &LoopNone;
	aFuncs[TODOFUNC_DEV_CHECK] = &CheckDevice;
	aFuncs[TODOFUNC_COM_CHECK] = &CheckComputer;
	aFuncs[TODOFUNC_DEBUG] = &LoopDebug;
	aFuncs[TODOFUNC_DEBUG_CHECK] = &CheckDebug;

	sett.dev_addr = 8;
	sett.computer = 0;
	sett.revert = 0;
	sett.select = 0;
	sett.debug = 0;

	ResetProto();
}


void	ResetProto()
{
	sett.revert = 0;
	sett.dorevert = 0;
	sett.select = 0;
	SwitchDirection( sett.computer );
	sett.EOI = 0;
	sett.SET_EOI = 0;
	sett.SET_LAST = 0;
	sett.ATN = 0;
	sett.ATNcount = 0;
	sett.data_send = 0;
	sett.error = 0;
	ResetCBM();
	LED_0 = 1;
	LED_1 = 0;
	LED_2 = 0;
	LED_3 = 0;
}


void	SwitchDirection( unsigned char sender )
{
	if( sett.debug )
	{
		sett.func = aFuncs[TODOFUNC_DEBUG];
		sett.check = aFuncs[TODOFUNC_DEBUG_CHECK];
	}
	else
	{
		if( 1 == sender ) {
			sett.func = aFuncs[TODOFUNC_DEV_SENDER + sett.computer];
		}
		else if( 2 == sender ) {
			sett.func = aFuncs[TODOFUNC_DEV_NONE + sett.computer];
		}
		else {
			sett.func = aFuncs[TODOFUNC_DEV_RECEIVER + sett.computer];
		}
		sett.check = aFuncs[TODOFUNC_DEV_CHECK + sett.computer];
	}
}


void	Loop()
{
	while( 1 )
	{
		sett.check();
		sett.func();
	}
}


void	LoopDebug()
{
	ReceiveUART( 0 );
	
	LED_0 = CBM_ATN_IN;
	LED_1 = CBM_CLK_IN;
	LED_2 = CBM_DTA_IN;
	LED_3 = CBM_RST_IN;
}


void    CheckDebug()
{
}


void	LoopNone()
{
	ReceiveUART( 0 );
}



unsigned char	ReceiveUART( unsigned char waitData, unsigned char* isData )
{
	unsigned char cmd;
	unsigned char val;
	
	if( waitData || RECV_CHECK_UART )
	{
		do
		{
			cmd = RECV_UART;
			SEND_UART( PC_RECEIVED );

			if( cmd == PC_DATA )
			{
				cmd = RECV_UART;
				SEND_UART( PC_RECEIVED );
				if( isData ) *isData = 1;
				return cmd;
			}
			else if( cmd == PC_EOI )
			{
				sett.SET_EOI = 1;
				SEND_UART( PC_EOI );
			}
			else if( cmd == PC_RST_ON )
			{
				SwitchDirection( 1 );
				CBM_RST_OUT( 1 );
				SEND_UART( PC_RST_ON );
			}
			else if( cmd == PC_RST_OFF )
			{
				CBM_RST_OUT( 0 );
				SEND_UART( PC_RST_OFF );
			}
			else if( cmd == PC_ATN_ON )
			{
				SwitchDirection( 1 );
				ZeroSender();
				CBM_ATN_OUT( 1 );
				sett.ATN = 1;
				sett.ATNcount = 0;
				SEND_UART( PC_ATN_ON );
			}
			else if( cmd == PC_ATN_OFF )
			{
				CBM_ATN_OUT( 0 );
				sett.ATN = 0;
				sett.ATNcount = 0;
				SEND_UART( PC_ATN_OFF );
			}
			else if( cmd == PC_DEV_ADDR )
			{
				cmd = RECV_UART;
				SEND_UART( PC_RECEIVED );
				sett.dev_addr = cmd;
				SEND_UART( PC_DEV_ADDR );
				SEND_UART( cmd );
			}
			else if( cmd == PC_RESET )
			{
				ResetProto();
				SEND_UART( PC_RESET );
			}
			else if( cmd == PC_DEVICE_MODE )
			{
				sett.computer = 0;
				sett.debug = 0;
				ResetProto();
				ZeroReceiver();
				SEND_UART( PC_DEVICE_MODE );
			}
			else if( cmd == PC_COMPUTER_MODE )
			{
				sett.computer = 1;
				sett.debug = 0;
				ResetProto();
				ZeroSender();
				SEND_UART( PC_COMPUTER_MODE );
			}
			else if( cmd == PC_DEBUG_MODE )
			{
				sett.computer = 0;
				sett.debug = 1;
				SwitchDirection( sett.computer );
				SEND_UART( PC_DEBUG_MODE );
			}
			else if( cmd == PC_STATUS )
			{
				SEND_UART( PC_STATUS );
				SEND_UART( 1 );	// version
				SEND_UART( sett.computer ? PC_COMPUTER_MODE : PC_DEVICE_MODE );
				SEND_UART( sett.dev_addr );
				SEND_UART( sett.error );
				SEND_UART( CBM_ATN_IN ? PC_ATN_ON : PC_ATN_OFF );
				SEND_UART( CBM_RST_IN ? PC_RST_ON : PC_RST_OFF );
				SEND_UART( CBM_DTA_IN ? PC_DTA_ON : PC_DTA_OFF );
				SEND_UART( CBM_CLK_IN ? PC_CLK_ON : PC_CLK_OFF );
			}
			else if( cmd == PC_TIMEOUT )
			{
				cmd = RECV_UART;
				SEND_UART( PC_RECEIVED );
				val = RECV_UART;
				SEND_UART( PC_RECEIVED );
				if( cmd == TIMEOUT_ALL ) {
					ResetTimeouts();
				}
				else {
					SetTimeouts( cmd, val );
				}
				SEND_UART( PC_TIMEOUT );
				SEND_UART( cmd );
			}
		}
		while( waitData );
	}
	return 0;
}







