/***********************************************************************/
/*                                                                     */
/*  FILE        :proto_computer.c                                      */
/*  DATE        :26.12.08                                              */
/*  COPYRIGHT   :Copyright (C) 2008 by Thomas Hochgtz                 */
/*                                                                     */
/***********************************************************************/
              
#include "proto_computer.h"
#include "proto.h"
#include "cbm.h"
#include "r8c.h"


#define DO_ERROR( err ) { sett.error = err; LED_3 = 1; SEND_UART(PC_ERROR); SEND_UART(err); }

void	CheckComputer()
{
	if( !sett.ATN )
	{
		if( sett.dorevert )
		{
			SwitchDirection( 0 );
			sett.dorevert = 0;
			if( RET_TIMEOUT == TurnSender() ) {
				DO_ERROR( PCERR_TURN );
			}
			sett.revert = 1;
		}
	}
}


void	ReceiverComputer()
{
	unsigned char ret;
	unsigned char data;
				
	if( !CBM_DTA_IN ) {
		ReceiveUART( 0 );
		Delay( 20 );
		return;
	}

	ret = ReadyToRecv();
	
	if( ret == RET_TIMEOUT_POLL )
	{
		ReceiveUART( 0 );
	}
	else if( ret == RET_TIMEOUT )
	{
		DO_ERROR( PCERR_READY_TO_RECV );
	}
	else
	{
		if( ret == RET_EOI )
		{
			if( RET_TIMEOUT == SendAckEOI() ) {
				DO_ERROR( PCERR_ACK_EOI );
			}
			sett.EOI = 1;
			SEND_UART( PC_EOI );
		}
		
		ret = RecvByte( &data );
		if( ret == RET_OK )
		{
			SendAckFrame();
			
			SEND_UART( PC_DATA );
			SEND_UART( data );
			
			if( sett.EOI )
			{
				ZeroReceiver();
				sett.EOI = 0;
				SwitchDirection( 2 );
			}
		}
	}
}


void	SenderComputer()
{
	unsigned char ret;
	unsigned char isData = 0;
	unsigned char data = 0;
	
	if( !sett.data_send )
	{
		data = ReceiveUART( 0, &isData );
		if( isData )
		{
			sett.data = data;
			sett.data_send = 1;
		}
		else
		{
			return;
		}
	}
		
	ret = ReadyToSend( sett.SET_EOI, (sett.ATN && 0==sett.ATNcount) );
	if( 0==sett.ATNcount ) sett.ATNcount = 1;
	
	if( ret == RET_TIMEOUT_POLL )
	{
	}
	else if( ret == RET_TIMEOUT )
	{
		DO_ERROR( PCERR_DEVICE_NOT_PRESENT );
		OffSender();
	}
	else if( ret == RET_EOI || ret == RET_OK )
	{
		if( ret == RET_EOI )
		{
			sett.SET_EOI = 0;
			sett.SET_LAST = 1;
			if( RET_TIMEOUT == RecvAckEOI() ) {
				DO_ERROR( PCERR_ACK_EOI );
			}
		}
		
		SendByte( sett.data );
		sett.data_send = 0;
	
		if( RET_TIMEOUT == RecvAckFrame() ) {
			DO_ERROR( PCERR_FRAME );
		}
	
		if( sett.ATN )
		{
			if( 1 == sett.ATNcount ) {
				CommandReceivedComputer( sett.data );
				sett.ATNcount = 2;
			}
		}
		
		if( sett.SET_LAST )
		{
			sett.SET_LAST = 0;
			SwitchDirection( 2 );
		}
	}
}


void	CommandReceivedComputer( unsigned char data )
{
	if( data >= PRG_LISTEN && data <= PRG_LISTEN_LAST )
	{
		sett.select = 1;
		LED_1 = 1;
		LED_2 = 1;
	}
	else if( data >= PRG_TALK && data <= PRG_TALK_LAST )
	{
		sett.select = 1;
		LED_1 = 1;
		sett.dorevert = 1;
	}
	else if( data == PRG_UNLISTEN )
	{
		sett.EOI = 0;
		sett.select = 0;
		LED_1 = 0;
		LED_2 = 0;
	}
	else if( data == PRG_UNTALK )
	{
		sett.EOI = 0;
		sett.select = 0;
		LED_1 = 0;
	}
}




